
const DynamicDropDownServices = require('../Services/DynamicDropDown.services');
const OrganizationServices = require('../Services/Organization.services');
const DynamicDropDown = require('../Model/DynamicDropDown');

/**
 * @description This is used for get list of all DropDowns
 * @param {*} req 
 * @param {*} res 
 * @returns object DynamicDropDownResult { status: true, dropdowns: Array }
 * - { status: false, error: string }
 */
module.exports.getAllDynamicDropDowns = async (req, res) => {
    try {

        let query = {};
        var count = 0;
        // query['organizationId'] = req.local.user.organization._id;
        const searchValue = req.query.searchValue;

        if (searchValue) {
            query['$or'] = [
                { name: { $regex: searchValue, $options: 'i' } },
                //     { "options.label": { $regex: searchValue, $options: 'i' } },
                //     { "options.name": { $regex: searchValue, $options: 'i' } },
            ]
        }

        const DynamicDropDownResult = await DynamicDropDownServices.getAllDynamicDropDowns(query, req.local.startIndex, req.local.limit);
        if (!DynamicDropDownResult.status || DynamicDropDownResult.error) return res.status(500).send(DynamicDropDownResult);

        const DynamicDropDownResultCount = await DynamicDropDownServices.getCount(query);
        if (!DynamicDropDownResultCount.status || DynamicDropDownResultCount.error) return res.status(500).send(DynamicDropDownResultCount);

        count = DynamicDropDownResultCount.dropdowns;

        // console.log(count, "count")

        return res.status(200).send({ status: true, dropdowns: DynamicDropDownResult.dropdowns, count: count });
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for get one DropDown
 * @param {*} req.params.name from that get name of the dropdown
 * @param {*} res 
 * @returns object DynamicDropDownResult { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.getDynamicDropDown = async (req, res) => {
    try {
        const name = req.params.name;
        if (!(name && name.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const DynamicDropDownResult = await DynamicDropDownServices.getDynamicDropDown({ name: name });
        if (!DynamicDropDownResult.status || DynamicDropDownResult.error) return res.status(500).send(DynamicDropDownResult);

        return res.status(200).send(DynamicDropDownResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for get report type list
 * @param {*} req.params.name from that get name of the dropdown
 * @param {*} res 
 * @returns object DynamicDropDownResult { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.getReportTypeList = async (req, res) => {
    try {
        const name = "type_of_report_list";
        if (!(name && name.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const DynamicDropDownResult = await DynamicDropDown.findOne({ name: name })
            .then((DynamicDropDowns) => {
                return { status: true, dropdowns: DynamicDropDowns };
            }).catch((e) => {
                console.log(e);
                return { status: false, error: e };
            });

        let result = "<items>";
        await Promise.all(DynamicDropDownResult.dropdowns.options.map(async data => {
            result += `<item label="${data.name}" />`
        }))
        result += "</items>"
        // console.log(result);
        return res.status(200).send(result);


        return res.status(200).send(DynamicDropDownResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for get construction type list
 * @param {*} req.params.name from that get name of the dropdown
 * @param {*} res 
 * @returns object DynamicDropDownResult { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.getConstructionType = async (req, res) => {
    try {
        const name = "construction_work_list";
        if (!(name && name.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const DynamicDropDownResult = await DynamicDropDown.findOne({ name: name })
            .then((DynamicDropDowns) => {
                return { status: true, dropdowns: DynamicDropDowns };
            }).catch((e) => {
                console.log(e);
                return { status: false, error: e };
            });


        let result = "<items>";
        await Promise.all(DynamicDropDownResult.dropdowns.options.map(async data => {
            result += `<item label="${data.name}" />`
        }))
        result += "</items >"
        // console.log(result);
        return res.status(200).send(result);


        return res.status(200).send(DynamicDropDownResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * This is used for create new Dropdown
 * @param {*} req.local.dropdown from that get dropDown Object 
 * @param {*} res 
 * @returns Object DynamicDropDown { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.createDynamicDropDown = async (req, res) => {
    try {
        var data = req.local.dropDown;

        var id = req.local.user._id;

        const alredyCreated = await DynamicDropDownServices.getAllDynamicDropDowns({ name: data.name })
        if (!alredyCreated.status || alredyCreated.error) return res.status(500).send(alredyCreated);

        if (alredyCreated.dropdowns.length == 0) {

            data.options.forEach((value) => {
                value['createBy'] = id;
                value['createdAt'] = new Date();
            })

            data.organizationId = req.local.user.organization._id;

            const DynamicDropDown = await DynamicDropDownServices.createDynamicDropDown(data);

            if (!DynamicDropDown.status) return res.status(500).send(DynamicDropDown);
            return res.status(200).send(DynamicDropDown);

        }
        else {
            return res.status(200).send("Already Created !");
        }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * This is used for update dropdown
 * @param {*} req.params.id from that get dropdownId and from req.local.dropdown get dropDown Object 
 * @param {*} res 
 * @returns Object DynamicDropDown { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.updateDynamicDropDown = async (req, res) => {
    try {
        const name = req.params.name;
        // console.log(name);
        var data = req.body;
        var id = req.local.user._id;

        data.organizationId = req.local.user.organization._id;
        // console.log(data);
        // const dropDown_res = await DynamicDropDownServices.getDynamicDropDown({ name: name });

        data.options.forEach((value) => {
            value['createBy'] = id;
            value['createdAt'] = new Date();
        })

        const DynamicDropDown = await DynamicDropDownServices.editDynamicDropDown(name, data);

        if (!DynamicDropDown.status) return res.status(500).send(DynamicDropDown);

        return res.status(200).send(DynamicDropDown);
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This function is used for push option object into option array i.e. get from database.
 * @param {*} req.params from that get dropdowns name and from req.body get option object
 * @param {*} res 
 * @returns Object DynamicDropDown { status: true, dropdowns: Object }
 * - { status: false, error: string }
 */
module.exports.editDynamicDropDown = async (req, res) => {
    try {
        const name = req.params.name;
        // console.log(name)
        var data = req.body;
        var id = req.local.user._id;

        // console.log(data,"---- body-----")
        data.createBy = id
        data.createdAt = new Date();

        const dropDown_res = await DynamicDropDownServices.getDynamicDropDown({ name: name });
        if (!dropDown_res.status) return res.status(500).send(dropDown_res);

        // console.log(dropDown_res.dropdowns.options)

        var options = dropDown_res.dropdowns.options;

        options.push(data);
        // console.log(options,"---- after push-----")
        const DynamicDropDown = await DynamicDropDownServices.editDynamicDropDown(name, { options: options });

        if (!DynamicDropDown.status) return res.status(500).send(DynamicDropDown);

        return res.status(200).send(DynamicDropDown);
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.deleteDynamicDropDown = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};