const bcrypt = require('bcrypt');
const fs = require('fs')
const jwt = require('jsonwebtoken');
const path = require('path');

const CONFIG = require('../Constants/Config');
const constpermission = require('../Constants/permission');

const { PermissionToBinary } = require('../Services/PermissionToBinaryServices');
const { workTypeArr } = require('../Types/Task');

const UserServices = require('../Services/User.services');
const RoleServices = require('../Services/Role.services');
const OrganizationServices = require('../Services/Organization.services');
const CaseServices = require('../Services/Case.services');
const BranchServices = require('../Services/Branch.services');
const ZoneServices = require('../Services/Zone.services');
const EmpanelmentServices = require('../Services/CreateServices')('Empanelment');
const AssetServices = require('../Services/Asset.services');
const MailController = require('../Controllers/Mail.controller');
const utilitiesController = require('../Controllers/Utilities.controller');

/**
 * @description This is used for get list of all users
 * @param {*} req
 * @param {*} res
 * @returns Object UserResult { status: true, users: Array }
 * - { status : false, error : string }
 */
module.exports.getAllUsers = async (req, res) => {
    try {
        const orgID = req.local.user.organization._id;
        const id = req.local.user._id;
        const { role, authUser, searchValue, orgBranch, workType, status = true, isdelete = false } = req.query;
        var count = 0;
        const type = "EMPLOYEE";
        let query = {
            isdelete,
            status,
            type,
            // 'organization._id' = { $in: [orgID] };
        };

        if (searchValue) {
            query["$or"] = [
                { firstName: { $regex: searchValue, $options: 'i' } },
                { lastName: { $regex: searchValue, $options: 'i' } },
                { role: { $regex: searchValue, $options: 'i' } },
                { designation: { $regex: searchValue, $options: 'i' } },
                { "goodPropertyType.label": { $regex: searchValue, $options: 'i' } }
            ]
        }
        if (authUser) { query['_id'] = { $ne: id }; }
        if (orgBranch) { query['orgBranches._id'] = orgBranch; }
        if (workType) { query['orgBranches.workTypes'] = { $in: [workType] } }

        const UserResult = await UserServices.findAll(query, req.local.limit, req.local.startIndex);
        if (!UserResult.status || UserResult.error) return res.status(500).send(UserResult);

        const UserResultCountSearch = await UserServices.getCount(query);
        if (!UserResultCountSearch.status || UserResultCountSearch.error) return res.status(500).send(UserResultCountSearch);
        count = UserResultCountSearch.users;

        if (role) {
            if (role == "VALUER") {
                // "organization._id": orgID,
                const UserResult = await UserServices.findAll({ role: "VALUER" });
                if (!UserResult.status || UserResult.error) return res.status(500).send(UserResult);

                return res.status(200).send(UserResult);
            }

            const RoleResult = await RoleServices.findByRoleName({ name: role });

            if (!RoleResult.status || RoleResult.error) return res.status(500).send(RoleResult);

            var rolePermission = RoleResult.roles.permission;

            var users = UserResult.users;
            let newUser = [];
            await users.map((user) => {
                let flag = true;
                for (property in user.permission) {
                    // console.log(user.permission)
                    for (key in rolePermission) {
                        if (key == property) {
                            for (i = 0; i < rolePermission[key].length; i++) {

                                if (rolePermission[key][i] != user.permission[key][i]) {
                                    if (user.permission[key][i] != '1') {

                                        flag = false;
                                    } else {
                                        // this else part written by Ashish to resolve blank pickup person list issue in case. And not checked other modules which is used this API and method, PLEASE CHECK OTHER MODULES and if you got error then feel free to remove this else part but it will reproduce blank pickup person list issue.
                                        flag = true;
                                    }
                                } else {
                                    // this else part written by Ashish to resolve blank pickup person list issue in case. And not checked other modules which is used this API and method, PLEASE CHECK OTHER MODULES and if you got error then feel free to remove this else part but it will reproduce blank pickup person list issue.
                                    flag = true;
                                }
                            }
                        }
                    }
                }
                if (flag) {
                    // console.log(user.name)
                    newUser.push(user);
                }
            })

            return res.status(200).send({ status: true, users: newUser });
        }
        else {
            return res.status(200).send({ status: true, users: UserResult.users, count: count });
        }

    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for get one user
 * @param {*} req.params.id from that get userId
 * @param {*} res
 * @returns Object UserResult { status: true, users: object }
 * - { status : false, error : string }
 */
module.exports.getUser = async (req, res) => {
    try {
        const id = req.params.id;
        // console.log(id, " type: ", typeof id)

        const loginRole = req.local.user.role;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const UserResult = await UserServices.findById(id);
        if (!UserResult.status || UserResult.error) return res.status(500).send(UserResult);

        // console.log(UserResult.users.role)
        if (UserResult.users.role == 'VALUER') {

            const empanelmentDetails = await EmpanelmentServices.find({ query: { valuerId: id, isDelete: false }, limit: 1000, populate: ['zoneId', 'valuerId'] });
            if (!empanelmentDetails.status || empanelmentDetails.error) return res.status(500).send(empanelmentDetails);

            const user_res = UserResult.users.toJSON();
            user_res.empanelments = empanelmentDetails.data;
            // console.log(JSON.stringify(empanelmentDetails.Empanelment))
            return res.status(200).send({ status: true, users: user_res });
        }

        const role = UserResult.users.role;
        if (loginRole == "HR" && role == "HR" || loginRole == "HR" && role == "ADMIN") {
            return res.status(200).send({ status: true, users: UserResult.users, permission: false });
        }

        return res.status(200).send(UserResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for create new user
 * @param {*} req.local.users from that get user Object
 * @param {*} res
 * @returns Object user_res { status: true, users: object }
 * - { status : false, err : string }
 */
module.exports.createUser = async (req, res) => {
    try {
        const datas = req.local.users;
        let orgId = datas.organization || req.local.user.organization;

        const userRes = await UserServices.getUserByEmail(datas.email)
        if (!userRes.users) return res.status(500).send({ status: false, userRes });

        const user_shortName = await UserServices.findAll({ shortName: datas.shortName })
        if (!user_shortName.users) return res.status(500).send({ status: false, user_shortName });

        if (user_shortName.users.length !== 0) {
            return res.status(500).send({ status: false, shortNameError: "ShortName Already Assigned!" });
        }
        if (userRes.users.length == 0) {

            // For Remove Image
            if (datas.removeImg) {
                var rmImgArr = [];
                datas.removeImg.map(async (path, id) => {

                    if (fs.existsSync(process.cwd() + "/" + path)) {
                        console.log("unlink img")
                        await fs.unlinkSync(process.cwd() + "/" + path);
                    }
                    else {
                        rmImgArr.push(path);
                    }
                });
                if (rmImgArr.length != 0) {
                    return res.send({ status: false, error: rmImgArr + " path doesn't exist For Remove Image" })
                }
            }
            delete datas.removeImg;

            if (datas.branch) {
                const Branch = await BranchServices.getBranch(datas.branch);
                if (!Branch.status) return res.status(500).send(Branch);

                datas.branch = Branch.branches
            }

            if (datas.permissions) {
                // ===== check for permissions object in request and convert to binary ie.{case:0101, roles: 0110} =====
                var res_result = await PermissionToBinary(datas.permissions);
                datas.permission = res_result.result;
            }

            const Organization = await OrganizationServices.getById(orgId);
            if (!Organization.status) return res.status(500).send(Organization);
            datas.organization = Organization.Organizations;

            if (!!datas.orgBranches) {
                let branchPermission = datas.orgBranches;
                datas.orgBranches = await Promise.all(Object.keys(branchPermission).map(async (_id, index) => {
                    const ob = branchPermission[_id];
                    const Organization = await OrganizationServices.getById(_id);
                    if (!Organization.status) return res.status(500).send(Organization);

                    return { ...Organization.Organizations.toJSON(), workTypes: ob };
                })).then(obs => {
                    return obs;
                });
            }

            const hashPassword = await UserServices.hashPassword(datas.password);
            if (!hashPassword) return res.status(500).send({ status: false, error: 'Internal Server Error!' });

            datas.password = hashPassword.hashPassword;

            const user_res = await UserServices.createUser(datas);
            if (!user_res.status) return res.status(500).send(user_res);

            const UserResult = user_res.users;
            const id = UserResult._id.toString();
            // console.log(id);
            if (req.file) {
                datas.profilePhoto = req.file.path;

                if (!fs.existsSync(process.env.CDN_PATH + '/users/' + id)) {
                    fs.mkdirSync(process.env.CDN_PATH + '/users/' + id, { recursive: true });
                }
                const fileName = "profilePhoto_" + req.file.filename;

                let currentPath = process.cwd() + "/" + req.file.path;
                const destination = path.join(process.env.CDN_PATH, "users", id, fileName);

                // call function for move file
                await utilitiesController.moveFile(currentPath, destination);

                const filepath = path.join("users/", id, fileName);
                UserResult.profilePhoto = filepath;
                UserResult.save();
            }
            if (UserResult.employeeDocs.length > 0) {

                if (!fs.existsSync(process.env.CDN_PATH + '/users')) {
                    fs.mkdirSync(process.env.CDN_PATH + '/users');
                }

                if (!fs.existsSync(process.env.CDN_PATH + '/users/' + id)) {
                    fs.mkdirSync(process.env.CDN_PATH + '/users/' + id);
                }

                const docsArray = UserResult.employeeDocs;
                // UserResult.employeeDocs = [];
                await Promise.all(docsArray.map(async (img, index) => {
                    // console.log(img);
                    const empDocs = img.images;
                    UserResult.employeeDocs[index].images = [];
                    await Promise.all(empDocs.map(async (file) => {
                        // console.log("FILE : ", file)
                        file.name = "employeeDocs_" + file.name;
                        const fileName = file.name;
                        let currentPath = process.cwd() + "/" + file.path;
                        const destination = path.join(process.env.CDN_PATH, "users", id, fileName);

                        // call function for move file
                        await utilitiesController.moveFile(currentPath, destination).then(async mvres => {

                            file.path = path.join("users", id, fileName);
                            if (UserResult.employeeDocs[index].images) {
                                // console.log(file, " ======== IF ===========")
                                await UserResult.employeeDocs[index].images.push(file);
                                // console.log("In IF", UserResult.employeeDocs[index])
                            }
                        }).catch(err => {
                            console.log("Error in Catch", err);
                        })
                    }))
                }))

                // console.log("Before save +++++++++++++++", JSON.stringify(UserResult),"======================")

                const user_res = await UserServices.editUser(id, UserResult);
                if (!user_res.status) return res.status(500).send(user_res);

                return res.status(200).send(user_res);
            }
            // console.log("Create User :", JSON.stringify(UserResult))
            return res.status(200).send({ status: true, users: UserResult });
        }
        else {
            return res.status(500).send({ status: false, error: "User already registered!" });
        }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.modifyUserStatus = async (req, res) => {
    try {
        const id = req.params.id;
        const active = req.query.active;

        const user_res = await UserServices.editUser(id, { status: active });
        if (!user_res.status) return res.status(500).send(user_res);

        return res.status(200).send(user_res);
    }
    catch (err) {
        console.log("🚀 ~ file: User.controller.js:324 ~ module.exports.userAccessModify= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
}

module.exports.deleteUser = async (req, res) => {
    try {
        const id = req.params.id;

        const user_res = await UserServices.editUser(id, { isdelete: true, status: false });
        if (!user_res.status) return res.status(500).send(user_res);

        return res.status(200).send(user_res);
    }
    catch (err) {
        console.log("🚀 ~ file: User.controller.js:324 ~ module.exports.userAccessModify= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
}

/**
 * @description This is used for update user
 * @param {*} req.params.id from that get userId and from req.local.users get user Object
 * @param {*} res
 * @returns Object user_res { status: true, users: object }
 * - { status : false, err : string }
 */
module.exports.updateUser = async (req, res) => {
    try {
        const id = req.params.id;
        const datas = req.local.users;
        // const orgID = req.local.user.organization;

        const UserResult = await UserServices.findById(id);
        if (!UserResult.status || UserResult.error) return res.status(500).send(UserResult);

        if (datas.permissions) {
            var res_result = await PermissionToBinary(datas.permissions);
            datas.permission = res_result.result;
        }

        if (req.file) {
            datas.profilePhoto = req.file.path;

            if (!fs.existsSync(process.env.CDN_PATH + '/users/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/users/' + id, { recursive: true });
            }

            console.log("🚀 ~ file: User.controller.js:345 ~ module.exports.updateUser= ~ req.file.path:", req.file.path)
            const fileName = "profilePhoto_" + req.file.filename;
            let currentPath = process.cwd() + "/" + req.file.path;
            const destination = path.join(process.env.CDN_PATH, "users", id, fileName);
            console.log("🚀 ~ file: User.controller.js:346 ~ module.exports.updateUser= ~ destination:", destination)
            // call function for move file
            await utilitiesController.moveFile(currentPath, destination);

            const filepath = path.join("users", id, fileName);
            datas.profilePhoto = filepath;
        }

        if (!!datas.organization) {
            const Organization = await OrganizationServices.getById(datas.organization);
            if (!Organization.status) return res.status(500).send(Organization);
            datas.organization = Organization.Organizations.toJSON();
        }

        if (!!datas.orgBranches) {
            let branchPermission = datas.orgBranches;
            datas.orgBranches = await Promise.all(Object.keys(branchPermission).map(async (_id, index) => {
                const ob = branchPermission[_id];
                const Organization = await OrganizationServices.getById(_id);
                if (!Organization.status) return res.status(500).send(Organization);

                return { ...Organization.Organizations.toJSON(), workTypes: ob };
            })).then(obs => {
                return obs;
            });
        }

        // For Remove Image
        if (datas.removeImg) {
            var rmImgArr = [];
            datas.removeImg.map(async (path, id) => {
                if (fs.existsSync(process.cwd() + "/" + path)) {
                    await fs.unlinkSync(process.cwd() + "/" + path);
                }
                else { rmImgArr.push(path); }
            });
        }
        delete datas.removeImg;

        if (datas.branch) {
            const Branch = await BranchServices.getBranch(datas.branch);
            if (!Branch.status) return res.status(500).send(Branch);

            datas.branch = Branch.branches
        }
        if (datas.password) {
            const hashPassword = await UserServices.hashPassword(datas.password);
            if (!hashPassword) return res.status(500).send({ status: false, error: 'Internal Server Error!' });

            // console.log(hashPassword)

            datas.password = hashPassword.hashPassword;
        }

        const user_res = await UserServices.editUser(id, datas);
        if (!user_res.status) return res.status(500).send(user_res);

        const usr = user_res.users;

        if (usr.role == "PICKUPPERSON") {
            const branch_pickupPerson = await BranchServices.editMultipleBranch({ "pickupPerson._id": id }, { pickupPerson: usr });
            if (!branch_pickupPerson.status) return res.status(500).send(branch_pickupPerson);

            const case_pickupPerson = await CaseServices.updateMultipleCase({ "staff.pickupPerson._id": id }, { "staff.pickupPerson": usr });
            if (!case_pickupPerson.status) return res.status(500).send(case_pickupPerson);
        }

        if (usr.role == "MAKER") {
            const case_maker = await CaseServices.updateMultipleCase({ "staff.maker._id": id }, { "staff.maker": usr });
            if (!case_maker.status) return res.status(500).send(case_maker);

        }
        if (usr.role == "CHECKER") {
            const case_checker = await CaseServices.updateMultipleCase({ "staff.checker._id": id }, { "staff.checker": usr });
            if (!case_checker.status) return res.status(500).send(case_checker);

        }
        if (usr.role == "SITEENGINEER") {
            const case_siteEngineer = await CaseServices.updateMultipleCase({ "staff.siteEngineer._id": id }, { "staff.siteEngineer": usr });
            if (!case_siteEngineer.status) return res.status(500).send(case_siteEngineer);
        }

        if (usr.role == "RATEVERIFIER") {
            const case_rateVerifier = await CaseServices.updateMultipleCase({ "staff.rateVerifier._id": id }, { "staff.rateVerifier": usr });
            if (!case_rateVerifier.status) return res.status(500).send(case_rateVerifier);
        }
        if (usr.role == "VALUER") {

            // const zone_valuer = await ZoneServices.updateMultipleZone({ "valuers._id": id }, { "$set": { "valuers.$[element]": usr } }, { "arrayFilters": [{ "element._id": id }] });
            // if (!zone_valuer.status) return res.status(500).send(zone_valuer);

            const zone_valuer = await ZoneServices.updateMultipleZone({ "valuers._id": id }, {
                "$set": {
                    "valuers.$.firstName": usr.firstName,
                    "valuers.$.lastName": usr.lastName,
                    "valuers.$.shortName": usr.shortName,
                    "valuers.$.fatherName": usr.fatherName,
                    "valuers.$.PANNo": usr.PANNo,
                    "valuers.$.profilePhoto": usr.profilePhoto,
                    "valuers.$.email": usr.email,
                    "valuers.$.designation": usr.designation,
                    "valuers.$.mobileNo": usr.mobileNo,
                    "valuers.$.role": usr.role,
                    "valuers.$.type": usr.type,
                    "valuers.$.registrationNo": usr.registrationNo,
                    "valuers.$.workWithDocumentMaker": usr.workWithDocumentMaker,
                }
            });
            if (!zone_valuer.status) return res.status(500).send(zone_valuer);

            const case_valuer = await CaseServices.updateValuerMultipleCase({ "staff.valuers._id": id }, {
                "$set": {
                    "staff.valuers.$.firstName": usr.firstName,
                    "staff.valuers.$.lastName": usr.lastName,
                    "staff.valuers.$.shortName": usr.shortName,
                    "staff.valuers.$.fatherName": usr.fatherName,
                    "staff.valuers.$.PANNo": usr.PANNo,
                    "staff.valuers.$.profilePhoto": usr.profilePhoto,
                    "staff.valuers.$.email": usr.email,
                    "staff.valuers.$.designation": usr.designation,
                    "staff.valuers.$.mobileNo": usr.mobileNo,
                    "staff.valuers.$.role": usr.role,
                    "staff.valuers.$.type": usr.type,
                    "staff.valuers.$.registrationNo": usr.registrationNo,
                    "staff.valuers.$.workWithDocumentMaker": usr.workWithDocumentMaker,
                }
            });

            if (!case_valuer.status) return res.status(500).send(case_valuer);
        }
        // console.log("Update User :", user_res)
        return res.status(200).send(user_res);
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

/**
 * @description This is used for hashing password
 * @param {*} req.params.pass from that get password
 * @param {*} res
 * @returns Object { status: true, password: string }
 */
module.exports.generatePassword = async (req, res) => {
    const password = req.params.pass;
    const hashPassword = await UserServices.hashPassword(password);
    if (!hashPassword) return res.status(500).send({ status: false, error: 'Internal Server Error!' });

    return res.status(200).send({ status: true, password: hashPassword });
};

/**
 * @description This is used for change password and hashing that password
 * @param {*} req.local.user from that get login user details and from req.body get object
 * @param {*} res
 * @returns Object { status: true, password: string }
 * - { status: false, error: "Internal Server Error!" }
 */
module.exports.changePassword = async (req, res) => {
    try {
        const email = req.local.user.email;
        const id = req.local.user._id;

        const {oldPassword, newPassword} = req.body;
        
        const isMatch = await UserServices.matchPassword(id, oldPassword);
        if (!isMatch) return res.status(400).json({ status: false, message: "Incorrect Password !" });

        const hashPassword = await UserServices.hashPassword(newPassword);
        if (!hashPassword.status || hashPassword.error) return res.status(500).send(hashPassword);

        const user_res = await UserServices.editUser(id, { password: hashPassword.hashPassword, forceResetPassword: false });
        if (!user_res.status) return res.status(500).send(user_res);

        return res.status(200).send({ status: true, password: hashPassword });
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for update user profile by login User
 * @param {*} req.local.user._id from that get login user Id and from req.local.users get user Object
 * @param {*} res
 * @returns Object user_res { status: true, users: object }
 * - { status : false, err : string }
 */
module.exports.updateUserProfile = async (req, res) => {
    try {
        const id = "" + req.local.user._id;
        // const organization_id = req.local.user.organization._id;
        const datas = req.local.users;

        if (req.file) {
            datas.profilePhoto = req.file.path;

            if (!fs.existsSync(process.env.CDN_PATH + '/users/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/users/' + id, { recursive: true });
            }

            const fileName = "profilePhoto_" + req.file.filename;
            let currentPath = process.cwd() + "/" + req.file.path;
            const destination = path.join(process.env.CDN_PATH, "users", id, fileName);
            // call function for move file
            console.log("🚀 ~ file: User.controller.js:556 ~ module.exports.updateUserProfile= ~ currentPath, destination:", currentPath, destination)
            await utilitiesController.moveFile(currentPath, destination);

            const filepath = path.join("users", id, fileName);
            datas.profilePhoto = filepath;
        }

        // For Remove Image
        if (datas.removeImg) {
            var rmImgArr = [];
            datas.removeImg.map(async (path, id) => {
                if (fs.existsSync(process.cwd() + "/" + path)) { await fs.unlinkSync(process.cwd() + "/" + path); }
                else { rmImgArr.push(path); }
            });
        }
        delete datas.removeImg;

        const user_res = await UserServices.editUser(id, datas);
        if (!user_res.status) return res.status(500).send(user_res);
        console.log("🚀 ~ file: User.controller.js:594 ~ module.exports.updateUserProfile= ~ user_res:", user_res)

        const usr = user_res.users;

        if (usr.role == "PICKUPPERSON") {
            const branch_pickupPerson = await BranchServices.editMultipleBranch({ "pickupPerson._id": id }, { pickupPerson: usr });
            if (!branch_pickupPerson.status) return res.status(500).send(branch_pickupPerson);

            const case_pickupPerson = await CaseServices.updateMultipleCase({ "staff.pickupPerson._id": id }, { "staff.pickupPerson": usr });
            if (!case_pickupPerson.status) return res.status(500).send(case_pickupPerson);
        }

        if (usr.role == "MAKER") {
            const case_maker = await CaseServices.updateMultipleCase({ "staff.maker._id": id }, { "staff.maker": usr });
            if (!case_maker.status) return res.status(500).send(case_maker);

        }
        if (usr.role == "CHECKER") {
            const case_checker = await CaseServices.updateMultipleCase({ "staff.checker._id": id }, { "staff.checker": usr });
            if (!case_checker.status) return res.status(500).send(case_checker);

        }
        if (usr.role == "SITEENGINEER") {
            const case_siteEngineer = await CaseServices.updateMultipleCase({ "staff.siteEngineer._id": id }, { "staff.siteEngineer": usr });
            if (!case_siteEngineer.status) return res.status(500).send(case_siteEngineer);
        }

        if (usr.role == "RATEVERIFIER") {
            const case_rateVerifier = await CaseServices.updateMultipleCase({ "staff.rateVerifier._id": id }, { "staff.rateVerifier": usr });
            if (!case_rateVerifier.status) return res.status(500).send(case_rateVerifier);
        }
        if (usr.role == "VALUER") {

            // const zone_valuer = await ZoneServices.updateMultipleZone({ "valuers._id": id }, { "$set": { "valuers.$[element]": usr } }, { "arrayFilters": [{ "element._id": id }] });
            // if (!zone_valuer.status) return res.status(500).send(zone_valuer);

            const zone_valuer = await ZoneServices.updateMultipleZone({ "valuers._id": id }, {
                "$set": {
                    "valuers.$.firstName": usr.firstName,
                    "valuers.$.lastName": usr.lastName,
                    "valuers.$.shortName": usr.shortName,
                    "valuers.$.fatherName": usr.fatherName,
                    "valuers.$.PANNo": usr.PANNo,
                    "valuers.$.profilePhoto": usr.profilePhoto,
                    "valuers.$.email": usr.email,
                    "valuers.$.designation": usr.designation,
                    "valuers.$.mobileNo": usr.mobileNo,
                    "valuers.$.role": usr.role,
                    "valuers.$.type": usr.type,
                    "valuers.$.registrationNo": usr.registrationNo,
                    "valuers.$.workWithDocumentMaker": usr.workWithDocumentMaker,
                }
            });
            if (!zone_valuer.status) return res.status(500).send(zone_valuer);

            //     console.log(usr)
            const case_valuer = await CaseServices.updateValuerMultipleCase({ "staff.valuers._id": id }, {
                "$set": {
                    "staff.valuers.$.firstName": usr.firstName,
                    "staff.valuers.$.lastName": usr.lastName,
                    "staff.valuers.$.shortName": usr.shortName,
                    "staff.valuers.$.fatherName": usr.fatherName,
                    "staff.valuers.$.PANNo": usr.PANNo,
                    "staff.valuers.$.profilePhoto": usr.profilePhoto,
                    "staff.valuers.$.email": usr.email,
                    "staff.valuers.$.designation": usr.designation,
                    "staff.valuers.$.mobileNo": usr.mobileNo,
                    "staff.valuers.$.role": usr.role,
                    "staff.valuers.$.type": usr.type,
                    "staff.valuers.$.registrationNo": usr.registrationNo,
                    "staff.valuers.$.workWithDocumentMaker": usr.workWithDocumentMaker,
                }
            });

            if (!case_valuer.status) return res.status(500).send(case_valuer);
        }

        return res.status(200).send(user_res);
    }
    catch (err) {
        console.log("🚀 ~ file: User.controller.js:654 ~ module.exports.updateUserProfile= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
};

/**
 * @description This is used for check user is already exist or not
 * @param {*} req.body get email of user  .
 * @param {*} res
 * @returns Object { status: false, error:"Already Created!" }
 * - return res.status(200).send({ status: true });
 */
module.exports.checkUserAlreadyExist = async (req, res) => {
    try {
        const email = req.body.email;
        const userRes = await UserServices.findAll({ email })
        if (!userRes.users) return res.status(500).send({ status: false, userRes });

        if (userRes.users.length == 0 || email == "") {
            return res.status(200).send({ status: true });
        }
        else {
            return res.status(500).send({ status: false, error: "Already Created!" });
        }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

/**
 * @description This is used for check valuer's registration number is already exist or not
 * @param {*} req.params.id from that get valuers Id from req.body get regNo.
 * @param {*} res
 * @returns Object { status: false, error: string }
 * - return res.status(200).send({ status: true });
 */
module.exports.checkRegNoAlreadyExist = async (req, res) => {
    try {
        const regNo = req.body.regNo;

        const userRes = await UserServices.findAll({ role: "VALUER", registrationNo: { $elemMatch: { number: regNo } } })
        if (!userRes.users) return res.status(500).send({ status: false, userRes });

        if (userRes.users.length != 0) {
            return res.status(500).send({ status: false, error: "Valuer Already Exist On " + regNo + "  Registration Number" });
        }
        else {
            return res.status(200).send({ status: true });
        }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

/**
 * @description This is used for get Asset
 * @param {*} req.params.id from that get user Id from req.query.status get status.
 * @param {*} res
 * @returns  Object { status: false, error: string }
 * - return res.status(200).send({ status: true, assets });
 */
module.exports.assetManagement = async (req, res) => {
    try {
        const id = req.params.id;
        const orgId = req.local.user.organization._id;

        // , orgId: orgId 
        const AssetResult = await AssetServices.aggregateAsset({ userId: id, status: req.query.status });
        if (!AssetResult.status || AssetResult.error) return res.status(500).send(AssetResult);

        // console.log(AssetResult);

        return res.status(200).send(AssetResult);
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for share link for reset password
 * @param {*} req.body.email from that get Email ID
 * @param {*} res
 * @returns Object { status: true, result: string }
 * - { status: false, error: string }
 */
module.exports.resetPasswordLink = async (req, res) => {
    try {
        const email = req.body.email.toLowerCase();

        const userRes = await UserServices.findAll({ email });
        if (!userRes.users) return res.status(500).send({ status: false, userRes });

        if (userRes.users.length != 0) {

            const jwtToken = await UserServices.generateJWTTokenForResetPassword(email)
            if (!jwtToken) return res.status(500).send({ status: false, jwtToken });

            const user = await UserServices.updateUser({ email: email }, { resetPasswordToken: jwtToken })
            if (!user.users) return res.status(500).send({ status: false, user });

            const link = CONFIG.FRONTEND_URL + "/resetpassword?resetPasswordToken=" + jwtToken;

            const html = `<p>Hello</p>If you've lost your password or wish to reset it, use the link below to get started. You can reset your password from this link: <br> <a href = ${link}>Reset Password Link</a> </p>`;
            const subject = "Reset Password Link";
            const from = "RealValue";
            const to = email;
            const message = "";
            const attachments = "";

            const mailResult = await MailController.SendMail(from, to, subject, message, attachments, html);

            return res.status(200).send({ status: true, message: "Reset Password Link Sent to Your Registered Email. Password Link will Expired in 60 minutes." });
        }
        else {
            return res.status(500).send({ status: false, error: email + " Not Registered. Please Enter Registered Email" });
        }
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for reset new password and hashing that password
 * @param {*} req.body.newPassword from that get Password
 * @param {*} res
 * @returns Object { status: true, password: string }
 * - { status: false, error: "Internal Server Error!" }
 */
module.exports.resetPassword = async (req, res) => {
    try {
        const resetPasswordToken = req.query.resetPasswordToken;
        const newPassword = req.body.newPassword;
        let query = {};

        query['resetPasswordToken'] = resetPasswordToken;

        const findUser = await UserServices.findOne(query)
        if (!findUser.status) return res.status(500).send(findUser);

        const user = findUser.users;

        if (user != null) {

            jwt.verify(resetPasswordToken, CONFIG.secret, async function (err, decoded) {
                if (err) {
                    if (err.name === "TokenExpiredError") {
                        return res.status(403).send({ status: false, error: 'Link Expired.' });
                    }
                    else {
                        return res.status(403).send({ status: false, error: 'Invalid Link.' });
                    }
                }
                const hashPassword = await UserServices.hashPassword(newPassword);
                if (!hashPassword.status || hashPassword.error) return res.status(500).send(hashPassword);

                const user_res = await UserServices.editUser(user._id, { password: hashPassword.hashPassword });
                if (!user_res.status) return res.status(500).send(user_res);

                return res.status(200).send({ status: true, message: "Your Password Successfully Reset" });
            })
        }
        else {
            return res.status(500).send({ status: false, error: "User Not Found" })
        }
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description For reset organization's Admin Password
 * @param {*} req
 * @param {*} res
 * @returns Object { status: true, message: "Your Password Successfully Reset" }
 * { status: false, error: "User Not Found" }
 */
module.exports.resetAdminPassword = async (req, res) => {
    try {
        const newPassword = req.body.newPassword;
        let query = {};
        const email = req.local.user.email;

        query['email'] = email;
        // query['resetPassword'] = false;

        const findUser = await UserServices.findOne(query)
        if (!findUser.status) return res.status(500).send(findUser);

        const user = findUser.users;

        if (user != null) {

            const hashPassword = await UserServices.hashPassword(newPassword);
            if (!hashPassword.status || hashPassword.error) return res.status(500).send(hashPassword);

            query['password'] = hashPassword.hashPassword;
            query['resetPassword'] = true;

            const user_res = await UserServices.editUser(user._id, query);
            if (!user_res.status) return res.status(500).send(user_res);

            return res.status(200).send({ status: true, message: "Your Password Successfully Reset" });
        }
        else {
            return res.status(500).send({ status: false, error: "User Not Found" })
        }
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};


/**
 * @description This is used for change password and hashing that password
 * @param {*} req.local.user from that get login user details and from req.body get object
 * @param {*} res
 * @returns Object { status: true, password: string }
 * - { status: false, error: "Internal Server Error!" }
 */
module.exports.notificationToken = async (req, res) => {
    try {
        const id = req.local.user._id;
        const notificationToken = req.params.token || null;

        if (!notificationToken) return res.status(404).send({ status: false, message: "Invalid Token" });

        const userRes = await UserServices.editUser(id, { notificationToken });
        if (!userRes.status) return res.status(500).send(userRes);

        return res.status(200).send({ status: true, message: "Successfully Updated Notification Token" });
    }
    catch (e) {
        console.log("🚀 ~ file: User.controller.js:881 ~ module.exports.notificationToken= ~ e:", e)
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for change password and hashing that password
 * @param {*} req.local.user from that get login user details and from req.body get object
 * @param {*} res
 * @returns Object { status: true, password: string }
 * - { status: false, error: "Internal Server Error!" }
 */
module.exports.taskWorkTypes = async (req, res) => {
    try {
        const user = req.local.user;

        const orgBranchPermission = user.orgBranches;

        let obWorkTypes = orgBranchPermission.reduce((prev, curr) => _.union(prev, curr.workTypes), [])

        if (["ADMIN"].includes(user.role)) {
            obWorkTypes = workTypeArr;
        }

        return res.status(200).send({ status: true, taskWorkTypes: obWorkTypes });
    }
    catch (e) {
        console.log("🚀 ~ file: User.controller.js:970 ~ module.exports.taskWorkTypes= ~ e:", e)
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

module.exports.emailSuggestion = async (req, res) => {
    try {
        // const user = req.local.user;
        const { search = "" } = req.query;
        const emails = await UserServices.find({
            query: {
                "email": { $regex: search, $options: 'i' }
            },
            display: { email: 1, firstName: 1, lastName: 1 }
        });

        return res.status(200).send({ status: true, ...emails });
    }
    catch (e) {
        console.log("🚀 ~ file: User.controller.js:985 ~ module.exports.emailSuggestion= ~ e:", e)
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
