const fs = require('fs');
const path = require('path');
const utilitesController = require('../Controllers/Utilities.controller');

if (!fs.existsSync('./public')) {
    fs.mkdirSync('./public');
}

if (!fs.existsSync('./public/images')) {
    fs.mkdirSync('./public/images');
}

if (!fs.existsSync('./public/images/image')) {
    fs.mkdirSync('./public/images/image');
}
if (!fs.existsSync(process.env.CDN_PATH + '/cases')) {
    fs.mkdirSync(process.env.CDN_PATH + '/cases');
}
/**
 * @description This is used for convert base64 String to Image and this is call from utilitiesController
 * @param {String} req 
 * @param {*} res 
 * @param {*} next 
 * @returns Object { status: true, ImagePath: imgPath }
 * - { status: false, error: "Base64String not found" }
 */
module.exports.base64ToImage = async (req, res, next) => {
    try {
        const base64String = req.body.base64String;
        const id = req.query.id;
        const tagName = req.query.tagName;

        if (base64String) {

            var slash = '/';
            var semicolon = ';';

            var str = base64String;
            var index1 = str.indexOf(slash) + slash.length;

            var index2 = str.indexOf(semicolon);

            var mimeType = '';
            if (index1 != 0 && index2 != 0) {
                mimeType = str.substring(index1, index2);
            }

            // var base64Data = base64String.replace(/^data:image\/png;base64,/, " ");

            if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id);
            }
            // call strAfter function
            const base64Data = await this.strAfter(base64String, ",");

            const fileName = tagName + "_" + Date.now() + '_' + "Image." + mimeType;

            const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);
            // call function for move file 
            // await utilitesController.moveFile(currentPath, destination);
            const filepath = path.join("cases", id, fileName);

            await fs.writeFile(destination, base64Data, 'base64', function (err) {

                // console.log(err);
                if (err) {
                    console.log("Error In Base64tiImage Middleware", err);
                    return res.status(500).send({ status: false, error: err });
                }
            });

            // var readFile = await fs.readFileSync(fullfilename);
            // console.log(readFile.)

            return res.status(200).send({ status: true, ImagePath: filepath });

        }
        else {
            return res.status(500).send({ status: false, error: "Base64String not found" });
        }

    }
    catch (e) {
        console.log(e);
        return { status: false, error: { type: 'Convert_To_Image', message: 'Error in Convert Base64 String To Image' } };
    }
}
/**
 * @description Retrieve the Part After a Character in a String
 * @param {String} string 
 * @param {String} delimiter 
 * @returns string
 */
module.exports.strAfter = async (string, delimiter) => {
    if (delimiter === '') {
        return string
    }
    const substrings = string.split(delimiter)

    return substrings.length === 1
        ? string // delimiter is not part of the string
        : substrings.slice(1).join(delimiter)
}
