const mongoose = require('mongoose');
const Schema = mongoose.Schema;

const invoiceSchema = Schema({
    organization: {
        _id: {
            type: Schema.Types.ObjectId,
            ref: 'Organization',
        },
        name: String,
        shortName: String,
        email: String,
        address: String,
        city: String,
        phone: String,
    },
    case: {
        _id: {
            type: Schema.Types.ObjectId,
            ref: 'Case',
        },
        caseNumber: String,
        ownerName: String,
        clientType: {},
        typeOfReport: {},
        client: {
            _id: { type: Schema.Types.ObjectId, ref: 'Client' },
            type: { type: String },
            name: String,
            displayName: String,
            initial: String,
        },
        branch: {
            _id: { type: Schema.Types.ObjectId, ref: 'Branch' },
            name: String,
            email: String,
            ledgerName: String,
        },
        outward: {},
        gstNo: String,
    },
    name: { type: String },
    gstNo: { type: String },
    basic: { type: Number, default: 0 },
    ops: { type: Number, default: 0 },
    gst: { type: Number, default: 0 },
    ed: { type: Number },
    tds: { type: Number },
    invoices: [
        {
            _id: { type: String, default: Date.now },
            invoiceType: {
                type: String,
                default: "PROFORMA" //  PROFORMA, TAX
            },
            invoiceNo: { type: String },
            isAdvance: { type: Boolean, default: false },
            note: String,
            amount: { type: Number, default: 0 },
            invoiceFile: {
                _id: false,
                originalname: String,
                path: String,
                fileName: String,
                createdAt: String,
                size: String,
                documentType: {},
                uploadedBy: {
                    _id: {
                        type: Schema.Types.ObjectId,
                        ref: 'User',
                    },
                    firstName: String,
                    lastName: String,
                    profilePhoto: String,
                },
                uploadedAt: { type: Date },
            },
            requestedBy: {
                _id: {
                    type: Schema.Types.ObjectId,
                    ref: 'User',
                },
                firstName: String,
                lastName: String,
                profilePhoto: String,
            },
            requestedAt: { type: Date, },
            completedAt: { type: Date },
            deletedAt: { type: Date },
            paymentStatus: { type: String, default: "PENDING" },
            status: {
                type: String,
                default: 'OPEN' // OPEN, COMPLETED, CANCELLED 
            },
            voucher: {
                guid: { type: String },
                date: { type: String },
                dispatchDocNo: { type: String },
                voucher_type: { type: String },
                _voucher_type: { type: String },
                voucher_number: { type: String },
                voucher_amount: { type: Number },
                narration: { type: String },
                party_name: { type: String },
                _party_name: { type: String },
                place_of_supply: { type: String },
                is_invoice: { type: Number },
                is_accounting_voucher: { type: Number },
                is_inventory_voucher: { type: Number },
                is_order_voucher: { type: Number },
                accounting: [{}],
                againstRef: [{}],
                newRef: [{}]
            }
        }
    ],
    payments: [
        {
            _id: {
                type: Schema.Types.ObjectId,
                ref: 'Payment',
            },
            paymentSettlement: {},
            receivedAmount: { type: Number }
        }
    ],
    completedInvoices: [{ type: String }],
    totalAmount: { type: Number, default: 0 },
    invoicedAmount: { type: Number, default: 0 },
    receivedAmount: { type: Number, default: 0 },
    invoiceStatus: {
        type: String, default: "PENDING", // PENDING, PARTIAL_RECEIVED, RECEIVED
    },
    status: {
        type: Boolean,
        default: true
    }
}, { timestamps: true });

module.exports = mongoose.model('Invoice', invoiceSchema);