const mongoose = require('mongoose');

const Schema = mongoose.Schema;

const organizationSchema = Schema({
    name: { type: String },
    shortName: { type: String },
    logo: { type: String },
    email: { type: String },
    mobileNo: { type: String },

    city: { type: String },
    state: { type: String, },
    address: { type: String },
    gstNo: { type: String },
    panNo: { type: String },
    contact: {
        name: String,
        mobileNo: String,
        email: String,
    },
    owners: [{
        _id: { type: Schema.Types.ObjectId, ref: 'User' },
        firstName: String,
        lastName: String,
        email: String,
        mobileNo: String,
        ownershipShare: String
    }],
    bankDetails: {},
    remark: { type: String },
    documents: [
        {
            fileName: String,
            path: String,
            type: { type: String, enum: ['STAMP_IMAGE', 'WATERMARK_BG'] },
            createdBy: { type: Schema.Types.ObjectId, ref: 'User' }
        }
    ],
    reportSettings: {},
    location: {
        latitude: String,
        longitude: String
    },
    isTallyLinked: { type: Boolean, default: false },
    activeSubscription: {},
    subscriptions: [{}],
    organization: {
        _id: {
            type: String
        },
        name: {
            type: String
        },
        email: {
            type: String
        },
        gstNo: {
            type: String
        }
    },
    type: {
        type: String, //
        enum: ['ORGANIZATION', 'BRANCH'],
        default: 'BRANCH'
    },
    status: {
        type: Boolean,
        default: true
    },
    isDeleted: {
        type: Boolean,
        default: false
    },
}, { timestamps: true });

module.exports = mongoose.model('Organization', organizationSchema);