const mongoose = require('mongoose');
const Schema = mongoose.Schema;
const { userSchema } = require('./CommonSchema');

const invoiceSchema = Schema({
    organization: {
        _id: {
            type: Schema.Types.ObjectId,
            ref: 'Organization',
        },
        name: String,
        shortName: String,
        email: String,
        address: String,
        city: String,
        phone: String,
    },
    clientType: {},
    client: {
        _id: { type: Schema.Types.ObjectId, ref: 'Client' },
        type: { type: String },
        name: String,
        displayName: String,
        initial: String,
    },
    branch: {
        _id: { type: Schema.Types.ObjectId, ref: 'Branch' },
        name: String,
        email: String,
        ledgerName: String,
    },
    invoiceType: {
        type: String,
        default: "PROFORMA" //  PROFORMA, TAX, ADV_PROFORMA, ADV_TAX
    },
    taxInvoiceId: { type: Schema.Types.ObjectId, ref: 'nInvoice' },
    invoiceNo: { type: String },
    gstNo: { type: String },
    basic: { type: Number, default: 0 },
    ops: { type: Number, default: 0 },
    gst: { type: Number, default: 0 },
    ed: { type: Number },
    tds: { type: Number },
    subId: { type: String, default: Date.now },
    isAdvance: { type: Boolean, default: false },
    note: String,
    amount: { type: Number, default: 0 },
    cases: [{
        _id: {
            type: Schema.Types.ObjectId,
            ref: 'Case',
        },
        caseNumber: String,
        ownerName: String,
        clientType: {},
        typeOfReport: {},
        outward: {},
        propertyType: {},
        gstNo: String,
        fees: { type: String, default: '0' },
        isCompleted: { type: Boolean, default: false },
        invoicedAmount: { type: Number, default: 0 },
        receivedAmount: { type: String, default: '0' },
        pendingAmount: { type: String, default: '0' }
    }],
    invoiceFile: {
        _id: false,
        originalname: String,
        path: String,
        fileName: String,
        createdAt: String,
        size: String,
        documentType: {},
        uploadedBy: {
            _id: {
                type: Schema.Types.ObjectId,
                ref: 'User',
            },
            firstName: String,
            lastName: String,
            profilePhoto: String,
        },
        uploadedAt: { type: Date },
    },
    requestedAt: { type: Date, },
    requestedBy: userSchema,
    paymentStatus: { type: String, default: "PENDING", enum: ['PENDING', 'PARTIAL_RECEIVED', 'RECEIVED'] },
    voucher: {
        guid: { type: String },
        date: { type: String },
        dispatchDocNo: { type: String },
        voucher_type: { type: String },
        _voucher_type: { type: String },
        voucher_number: { type: String },
        voucher_amount: { type: Number },
        narration: { type: String },
        party_name: { type: String },
        _party_name: { type: String },
        place_of_supply: { type: String },
        is_invoice: { type: Number },
        is_accounting_voucher: { type: Number },
        is_inventory_voucher: { type: Number },
        is_order_voucher: { type: Number },
        accounting: [{}],
        againstRef: [{}],
        newRef: [{}]
    },
    completedCaseIds: [{ type: Schema.Types.ObjectId, ref: 'Case' }],
    payments: [
        {
            _id: {
                type: Schema.Types.ObjectId,
                ref: 'Payment',
            },
            paymentSettlement: {},
            receivedAmount: { type: Number }
        }
    ],
    accounting: [
        {
            _id: {}, // invoiceId
            voucherType: { type: String, default: "payment", enum: ["sales", "payment"] },
            paymentId: {
                type: Schema.Types.ObjectId,
                ref: 'Payment',
            },
            isCompleted: { type: Boolean, default: false },
            invoicedAmount: { type: Number, default: 0 },
            receivedAmount: { type: Number, default: 0 },
            pendingAmount: { type: Number, default: 0 },
            cases: [{
                _id: {
                    type: Schema.Types.ObjectId,
                    ref: 'Case'
                },
                fees: { type: String, default: '0' },
                isCompleted: { type: Boolean, default: false },
                invoicedAmount: { type: Number, default: 0 },
                receivedAmount: { type: Number, default: 0 },
                pendingAmount: { type: Number, default: 0 }
            }],
            completedAt: { type: Date },
            completedBy: userSchema
        }
    ],
    receivedAmount: { type: Number, default: 0 },
    pendingAmount: { type: Number, default: 0 },
    invoiceStatus: {
        type: String, default: "OPEN", // COMPLETED
    },
    review: [
        {
            _id: false,
            _id: { type: String, default: Date.now },
            note: { type: String },
            completedAt: { type: Date },
        }
    ],
    cancelledAt: { type: Date },
    cancelledBy: userSchema,
    completedAt: { type: Date },
    completedBy: userSchema,
    status: { type: Boolean, default: true }
}, { timestamps: true });

invoiceSchema.pre('save', async function (next) { // this line
    const invoice = this;
    console.log("inv pending amount: ", invoice.pendingAmount, invoice.amount, invoice.receivedAmount)
    invoice.pendingAmount = invoice.amount - invoice.receivedAmount;
    next();
});

module.exports = mongoose.model('nInvoice', invoiceSchema);