const CaseReportLog = require('../Model/CaseReportLog');
/**
 * @description This is used for get list of all CaseReportLogs 
 * @param {*} query 
 * @returns Object { status: true, CaseReportLogs: Array }
 * - { status: false, error: string }
 */
module.exports.getAllCaseReportLogs = async (query = {}) => {

    return CaseReportLog.find(query).then((CaseReportLogs) => {
        return { status: true, CaseReportLogs: CaseReportLogs };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get One CaseReportLog By Id
 * @param {String} id 
 * @returns Object { status: true, CaseReportLogs: Object }
 * - { status: false, error: string }
 */
module.exports.getCaseReportLog = async (id) => {

    return CaseReportLog.findById(id).then((CaseReportLog) => {
        return { status: true, CaseReportLogs: CaseReportLog };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get One CaseReportLog
 * @param {*} query 
 * @returns Object { status: true, CaseReportLogs: Object }
 * - { status: false, error: string }
 */
module.exports.findCaseReportLog = async (query) => {

    return CaseReportLog.findOne(query).then((CaseReportLog) => {
        return { status: true, CaseReportLogs: CaseReportLog };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for create CaseReportLog
 * @param {Object} CaseReportLogData 
 * @returns Object { status: true, CaseReportLogs: Object }
 * - { status: false, error: string }
 */
module.exports.createCaseReportLog = async (CaseReportLogData) => {

    return CaseReportLog.create(CaseReportLogData)
        .then((result) => {
            // console.log("create CaseReportLog Service")
            return { status: true, CaseReportLogs: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};
/**
 * @description This is used for update CaseReportLog
 * @param {String} id 
 * @param {Object} CaseReportLogData 
 * @returns Object { status: true, CaseReportLogs: Object }
 * - { status: false, error: string }
 */
module.exports.updateCaseReportLog = async (id, CaseReportLogData) => {

    return CaseReportLog.findOneAndUpdate({ _id: id }, CaseReportLogData)
        .then((result) => {
            return { status: true, CaseReportLogs: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};