const { Expo } = require('expo-server-sdk');

module.exports.sendExpoNotification = async (pushTokens = [], body = '', data = {}) => {

    try {
        let expo = new Expo();  // new Expo({ accessToken: process.env.EXPO_ACCESS_TOKEN });
        let messages = [];

        for (let pushToken of pushTokens) {
            if (!Expo.isExpoPushToken(pushToken)) { // Check that all your push tokens appear to be valid Expo push tokens
                console.error(`Push token ${pushToken} is not a valid Expo push token`);
                return { status: false, error: `Push token ${pushToken} is not a valid Expo push token` }
            }

            // TODO: Develop the code here to check this token is canceled or uninstalled app token
            messages.push({ to: pushToken, sound: 'default', body, data, });
        }

        // Send Notification
        let chunks = expo.chunkPushNotifications(messages);
        let tickets = [];
        for (let chunk of chunks) {
            try {
                let ticketChunk = await expo.sendPushNotificationsAsync(chunk);
                console.log("ticketChunk", ticketChunk);
                tickets.push(...ticketChunk);
            } catch (error) {
                console.error(error);
            }
        }

        // // receipt notification
        // let receiptIds = tickets.reduce((prev, curr) => !!curr.id ? [...prev, curr.id] : prev, []);

        // let receiptIdChunks = expo.chunkPushNotificationReceiptIds(receiptIds);
        // (async () => {
        //     for (let chunk of receiptIdChunks) {
        //         try {
        //             let receipts = await expo.getPushNotificationReceiptsAsync(chunk);
        //             console.log("receipts ", receipts);

        //             // The receipts specify whether Apple or Google successfully received the
        //             // notification and information about an error, if one occurred.
        //             for (let receiptId in receipts) {
        //                 let { status, message, details } = receipts[receiptId];
        //                 if (status === 'ok') {
        //                     console.log(receipts[receiptId], "Status ok")
        //                     continue;
        //                 } else if (status === 'error') {
        //                     console.error(`There was an error sending a notification: ${message}`);
        //                     if (details && details.error) {
        //                         console.error(`The error code is ${details.error}`);
        //                     }
        //                 }
        //             }
        //         } catch (error) {
        //             console.error(error);
        //         }
        //     }
        // })();

        let response = "";

        for (const ticket of tickets) {
            if (ticket.status === "error") {
                if (ticket.details && ticket.details.error === "DeviceNotRegistered") {
                    response = "DeviceNotRegistered";
                }
            }

            if (ticket.status === "ok") {
                response = ticket.id;
            }
        }

        console.log("🚀 ~ file: ExpoNotification.services.js:79 ~ module.exports.sendExpoNotification= ~ response:", response)
        return response;

    } catch (error) {
        console.log("Expo Error", error)
    }
}