const User = require('../Model/User');
const Task = require('../Model/Task');
const activeUsers = new Map();
const activeCases = new Map();
const _ = require('loadsh');

module.exports = function (io, socket) {

    io.on('connection', (socket) => {
        console.log("connected: ", socket.id);
    });

    function updateUser() {
        const uniqueById = _.uniqWith([...activeUsers.values()], _.isEqual);
        io.emit('ACTIVE_USERS', uniqueById);
    }

    function updateWork() {
        io.emit('ACTIVE_WORKS', Array.from(activeCases.values()));
    }

    socket.on('login', async function (data) {
        try {

            if (data.userId) {
                const user = await User.findById(data.userId, ['firstName', 'lastName', 'organization.name']).lean();
                activeUsers.set(socket.id, { ...user, platform: data.platform || "-" });

                updateUser();
                updateWork();
            }
        } catch (e) {
            console.log("🚀 ~ file: index.js:31 ~ e:", e)
        }
    });

    socket.on('START_WORK', async function (data) {
        try {
            if (data.taskId && data.workType) {
                if (!activeCases.has(socket.id)) {
                    activeCases.set(socket.id, data);
                    await Task.updateOne({ _id: data.taskId }, { '$push': { timeLog: { time: new Date(), type: "START" } } })
                    updateWork();
                }
            }
        } catch (e) {
            console.log("🚀 ~ file: index.js:31 ~ e:", e)
        }
    });

    socket.on('STOP_WORK', async function (data) {
        try {
            if (activeCases.has(socket.id)) {
                const { taskId } = activeCases.get(socket.id);
                activeCases.delete(socket.id);
                await Task.updateOne({ _id: taskId }, { '$push': { timeLog: { time: new Date(), type: "END" } } });
                updateWork();
            }
        } catch (e) {
            console.log("🚀 ~ file: index.js:62 ~ e:", e)
        }
    });

    socket.on('EXTERNAL_UPLOAD', function (data) {
        console.log("🚀 ~ file: SOcketIO index.js:15 ~ EXTERNAL_UPLOAD ~ data:", data)
        io.emit('EXTERNAL_UPLOAD_RES', data);
    });

    socket.on('CASE_STATUS_UPDATE', function (data) {
        console.log("🚀 ~ file: SOcketIO index.js:20 ~ data:", data)
        io.emit('CASE_STATUS_UPDATE_RES', data)
    });

    socket.on('disconnect', async function () {
        try {
            console.log("user disconnected");
            activeUsers.delete(socket.id);
            updateUser();
            
            if (activeCases.has(socket.id)) {
                let { taskId } = activeCases.get(socket.id);
                activeCases.delete(socket.id);
                !!taskId && await Task.updateOne({ _id: taskId }, { '$push': { timeLog: { time: new Date(), type: "END_CLOSE" } } });
                updateWork();
            }
        }
        catch (e) {
            console.log("🚀 ~ file: index.js:91 ~ e:", e)
        }
    });
};