const fs = require('fs');
const express = require("express");
const app = express();
const server = require('https').createServer({
    key: fs.readFileSync('./ssl/STAR.vastukala.co.in.key'),
    cert: fs.readFileSync('./ssl/STAR.vastukala.co.in.crt'),
    ca: [fs.readFileSync('./ssl/STAR.vastukala.co.in.ca-bundle')]
}, app);
const { createAdapter } = require("@socket.io/cluster-adapter");
const { setupWorker } = require("@socket.io/sticky");

const io = require("socket.io")(server, { cors: { origin: "*" } });
io.adapter(createAdapter());
setupWorker(io);
process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = 0;

const cors = require('cors');
const path = require('path');
const compression = require('compression');

const mongoose = require('mongoose');
mongoose.Promise = global.Promise;
const morgan = require('morgan');
const { logger } = require('./logger');

const dotenv = require('dotenv').config();
const CONFIG = require('./Constants/Config');

const RouteLoader = require('./routes/index');
const CronJobs = require('./cron');

const Task = require('./Model/Task');
const Case = require('./Model/Case');
const caseTask = require('./hooks/caseTask');

// app.use(morgan('combined'));
const morganMiddleware = morgan(
    function (tokens, req, res) {
        return JSON.stringify({
            remote_addr: tokens['remote-addr'](req, res),
            method: tokens.method(req, res),
            url: tokens.url(req, res),
            status: Number.parseFloat(tokens.status(req, res)),
            content_length: tokens.res(req, res, 'content-length'),
            response_time: Number.parseFloat(tokens['response-time'](req, res)),
        });
    },
    { stream: { write: (message) => { const data = JSON.parse(message); logger.http(`incoming-request`, data) } } }
);
app.use(morganMiddleware);

app.use(express.urlencoded({ extended: true }));
app.use(express.json({ limit: "5mb" }));

const allowedOrigins = ['https://staging.vastukala.co.in', 'staging.vastukala.co.in:5000', "https://api.real-value.co.in", "api.real-value.co.in", "http://localhost:5454"];

app.use(function (req, res, next) {
    req.headers.origin = req.headers.origin || req.headers.host;
    next();
});

app.options('*', cors());
// Enable CORS for all routes or specific routes by adding cors(corsOptions) to specific routes
app.use(cors({
    origin: (origin, callback) => {
        var originIsWhitelisted = allowedOrigins.indexOf(origin) !== -1
        callback(originIsWhitelisted ? null : 'Bad Request', originIsWhitelisted)
    },
    methods: ['GET', 'HEAD', 'PUT', 'PATCH', 'POST', 'DELETE'],
    credentials: true,
    allowedHeaders: '*',
    optionsSuccessStatus: 200,
    preflightContinue: true
}));

app.use(compression());

const onConnection = (socket) => { require('./SocketIO/index')(io, socket); }
io.on("connection", onConnection);

// Middleware to exclude a specific folder from static serving
app.use((req, res, next) => {
    const excludedFolder = '/cdn/sec_db_bk';
    if (req.url.startsWith(excludedFolder)) {
        logger.error(`🚀 ~ app.use ~ excludedFolder:`, excludedFolder)
        return res.status(400).send();
    } else { next(); }
});

app.use('/cdn', express.static(path.join(__dirname, '..', 'CDN')));
app.use('/public', express.static(path.join(__dirname, 'public')));
app.use('/docx', express.static(path.join(__dirname, 'docx')));
app.use('/utils', express.static(path.join(__dirname, 'Utils')));
app.get('/status', async (req, res) => { return res.status(200).json({ status: true }); });
// app.get('/g-id', async (req, res) => { return res.status(200).send({ id: "\n\n\n***********\nPM_ID = " + process?.env?.pm_id + " \n*********************\n\n\n" }); })

(async function () {
    const routes = await RouteLoader('./routes/*.routes.js');
    app.use(routes);
})()

if (process?.env?.pm_id) {
    if (process.env.pm_id == 2) {
        logger.error(`=> started db change listen on ${process.env.pm_id} node`)
        Task.watch().on('change', caseTask)
        Case.watch().on('change', caseTask)
    }
} else {
    logger.error("no cluster mode so listen db changes")
    Task.watch().on('change', caseTask)
    Case.watch().on('change', caseTask)
}


async function startServer() {
    mongoose
        .connect(CONFIG.MONGODB_URL, { useNewUrlParser: true, useUnifiedTopology: true, useFindAndModify: false })
        .then(result => {
            const PORT = process.env.PORT || 5000;
            server.listen(PORT, function () {
                console.log(`RealValueStaging app listening on port ${PORT} ! Go to https://staging.vastukala.co.in:${PORT}/`)
                CronJobs.startCronJobs()
            })
            server.timeout = 1500000;
        })
        .catch(err => {
            logger.error(err);
            console.log("error: ", err)
        });
}

// Run the async function to start our server
startServer();
