const { MongoClient, ObjectId } = require('mongodb');
const fs = require('fs');
const path = require('path');
const CDN = '../CDN/';
var args = process.argv.slice(2);

async function main() {

    const client = new MongoClient("mongodb://192.168.1.126:27017");
    try {
        // Connect to the MongoDB cluster
        await client.connect();
        const cases = client.db('realvalue').collection("cases");

        // Find the existing case document
        const caseNumber = args[0];
        const copyToCaseNumber = args[1];

        const existingCase = await cases.findOne({ caseNumber });

        if (!existingCase) {
            console.log(`No case found with caseNumber: ${caseNumber}`);
            return;
        }

        const copyToCase = await cases.findOne({ caseNumber: copyToCaseNumber });

        if (!copyToCase) {
            console.log(`No case found with caseNumber: ${copyToCaseNumber}`);
            return;
        }
        const copyToCaseId = copyToCase._id; 
        const oldId = existingCase._id;
        const newId = copyToCase._id; 
        // Duplicate the existing document with a new _id
        delete existingCase._id;
        delete existingCase.caseNumber;
        delete existingCase.createdAt;
        delete existingCase.updatedAt;
        delete existingCase.compositionFlatType;
        delete existingCase.otherRateDetails;
        const duplicatedCase = { ...existingCase };

        const updatedJsonObject = updateIdInJsonString(JSON.stringify(duplicatedCase), oldId, newId);

        // Update caseNumber and createdAt in the duplicated document
        updatedJsonObject.outward = { status: false };
        updatedJsonObject.invoiceId = null;

        // Insert the duplicated document into the collection
        await cases.findOneAndUpdate({_id:copyToCaseId},[{$set:updatedJsonObject}]);
        // console.log("=> new", {_id: copyToCaseId },updatedJsonObject);

        console.log(`Case ${caseNumber} copied successfully to caseNumber: ${copyToCase.caseNumber}`);

        // Copy the entire folder
        await copyFolderRecursive(CDN + 'cases/' + oldId, CDN + 'cases/' + newId);

    } catch (e) {
        console.error(e);
    } finally {
        await client.close();
    }
}

function updateIdInJsonString(jsonString, oldId, newId) {
    try {
        const updatedJsonString = jsonString.replace(new RegExp(`\\b${oldId}\\b`, 'gi'), newId);
        const jsonObject = JSON.parse(updatedJsonString);
        return jsonObject;
    } catch (error) {
        console.error('Error parsing or updating JSON:', error);
        return null;
    }
}

function copyFolderRecursive(sourcePath, destinationPath) {
    // Ensure the destination directory exists
    if (!fs.existsSync(destinationPath)) {
        fs.mkdirSync(destinationPath);
    }

    // Read the contents of the source directory
    const files = fs.readdirSync(sourcePath);
    console.log("->OLD: ", sourcePath, "\n New: ", destinationPath);

    files.forEach((file) => {
        const sourceFilePath = path.join(sourcePath, file);
        const destinationFilePath = path.join(destinationPath, file);
        // Check if the current item is a directory
        const isDirectory = fs.statSync(sourceFilePath).isDirectory();

        if (isDirectory) {
            // Recursively copy subdirectories
            copyFolderRecursive(sourceFilePath, destinationFilePath);
        } else {
            // Copy individual files
            fs.copyFileSync(sourceFilePath, destinationFilePath);
        }
    });
}

main().catch(console.error);