const cron = require('node-cron');
const { logger } = require('./logger');
const TaskServices = require('./Services/CreateServices')('Task');


function ClientVisitAutoComplete() {
    cron.schedule('*/30 * * * *', async () => {
        console.log('Running cron job...');
        try {
            const twoHoursAgo = new Date(Date.now() - 2 * 60 * 60 * 1000); 
            // Find documents where updatedAt is greater than 2 hours ago
            const documentsToUpdate = await TaskServices.find({ query: { workType: "CLIENT_VISIT", status: "MY_TASK", updatedAt: { $lt: twoHoursAgo } }, limit: 1000000 });
            console.log("query: ", { query: { workType: "CLIENT_VISIT", status: "MY_TASK", updatedAt: { $lt: twoHoursAgo } }, limit: 1000000 }, " - ", documentsToUpdate.count);
            if (documentsToUpdate.status) {
                
                for (const doc of documentsToUpdate.data) {
                    // complete each tasks which is open for more then 2 hr
                    doc.timeLog.push({
                        time: new Date(),
                        type: "END",
                        geo: { type: 'Point', coordinates: [0, 0] }
                    });
                    doc.remarks.push({
                        id: Math.floor(100000 + Math.random() * 900000),
                        remark: 'AUTO COMPETED BY SERVER.',
                        createdAt: new Date(),
                        creditOfficerId: null,
                        firstName: "-",
                        lastName: "-",
                        role: "-",
                    });
                    doc.status = "COMPLETED";
                    await doc.save();
                    console.log(`Updated document with ID ${doc._id}`);
                }
            }
        } catch (error) {
            logger.error('Error in cron job:' + error.toString());
        }
    });
}

function startCronJobs() {
    ClientVisitAutoComplete();
}

module.exports = { startCronJobs };
