const { MongoClient, ObjectId } = require('mongodb');

// MongoDB connection setup
const uri = 'mongodb://192.168.1.126:27017'; // Update with your MongoDB connection string
const dbName = 'realvalue'; // Replace with your database name
let client;

async function updatePayments() {
  try {
    // Connect to MongoDB
    client = new MongoClient(uri, { useNewUrlParser: true, useUnifiedTopology: true });
    await client.connect();
    db = client.db(dbName);
    console.log('Connected to MongoDB');

    const paymentsCollection = db.collection('payments');
    const invoicesCollection = db.collection('ninvoices');

    // Fetch payments with the required conditions
    const paymentResults = await paymentsCollection
      .find({
        "paymentSettlement.cases.receivedAmount": 0,
        paymentStatus: "COMPLETED",
        "paymentSettlement.1": { $exists: false },
      })
      .sort({createdAt:1})
      .limit(20)
      .toArray();

    if (!paymentResults || paymentResults.length === 0) {
      console.log('No payments found with the specified conditions.');
      return;
    }

    // Iterate over the fetched payments
    const responses = await Promise.all(
      paymentResults.map(async (pment) => {
        const paymentId = pment._id.toString();

        // Fetch related invoices for payment settlements
        const relatedInvoices = await invoicesCollection
          .find({ _id: { $in: pment.paymentSettlement.map((ps) => ObjectId(ps._id)) } })
          .toArray();
        console.log("query: ", { _id: { $in: pment.paymentSettlement.map((ps) => ObjectId(ps._id)) } });
        // Iterate over payment settlements to update receivedAmount
        for (const settlement of pment.paymentSettlement) {
          // Fetch related invoices for the settlement
          // Update receivedAmount for each case in the settlement
          const relatedInvoice = relatedInvoices.find(
            (invoice) => invoice._id.toString() === settlement._id
          );
          settlement.cases.forEach((caseItem) => {

            if (relatedInvoice) {
              // Update receivedAmount with invoice amount
              caseItem.invoicedAmount = +relatedInvoice.amount || 0;
              caseItem.receivedAmount = (+pment.voucher.voucher_amount + +pment.discount) || 0;
              caseItem.pendingAmount = (+relatedInvoice.amount || 0) - ((+pment.voucher.voucher_amount + +pment.discount) || 0);
              caseItem.isCompleted = !!(caseItem.pendingAmount == 0) || false;
              console.log("in related Invoice ia: ",relatedInvoice.amount,"ra: ",(+pment.voucher.voucher_amount + +pment.discount), "pa: ",caseItem.pendingAmount, "dis: ",+pment.discount);
            }
          });
          settlement.receivedAmount = settlement.cases.reduce((acc, c) => acc + (+c.receivedAmount || 0), 0);
          settlement.invoicedAmount = settlement.cases.reduce((acc, c) => acc + (+c.invoicedAmount || 0), 0);
          settlement.pendingAmount = ( settlement.invoicedAmount - settlement.receivedAmount) || 0;
          settlement.isCompleted = !!(settlement.pendingAmount == 0) || false;
        }

        // Save the updated payment document
        await paymentsCollection.updateOne(
          { _id: ObjectId(paymentId) },
          { $set: { paymentSettlement: pment.paymentSettlement } }
        );

        return { status: true, message: `Payment updated for ID: ${paymentId}` };
      })
    );

    // Log the results
    responses.forEach((response) => {
      if (response.status) {
        console.log(response.message);
      } else {
        console.log(`Error updating payment for ID: ${response.id}`);
      }
    });
  } catch (error) {
    console.error('Error occurred during payment update:', error);
  } finally {
    // Close the MongoDB connection
    if (client) {
      client.close();
      console.log('Disconnected from MongoDB');
    }
  }
}

// Run the updatePayments function
updatePayments();
