const express = require('express');
const router = express.Router();
const BranchController = require('../Controllers/Branch.controller');
const CreditOfficerController = require('../Controllers/CreditOfficer.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const isPermission = require('../Middleware/isPermission');
const pagination = require('../Middleware/pagination');
const uploadFile = require('../Middleware/uploadFile');
const validation = require('../Middleware/validation');

// For list all branches 
router.get('/Branches', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "branch", value: "list" }) },pagination, BranchController.getAllBranches);

// For list of all branches that have No Zone from ClientId 
router.get('/BranchesWithoutZone/:id', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "branch", value: "list" }) }, BranchController.getAllBranchesWithoutZone);

// For get one Branch 
router.get('/Branches/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "branch", value: "view" }) }, BranchController.getBranch);

// For create new Branch 
router.post('/Branches', isAuth, isAdmin, validation.branchValidation, BranchController.createBranch);

// For update Branch 
router.put('/Branches/:id', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "branch", value: "edit" }) }, validation.branchValidation, BranchController.updateBranch);

// For list Credit officer Branch wise 
router.get('/Branches/:id/creditofficers', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "list" }) }, BranchController.getAllCreditOfficersByBranch);

// For get list of deactivated Credit officer Branch wise 
router.get('/Branches/:id/deactivatedCreditofficers', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "list" }) }, BranchController.getAllDeactivatedCreditOfficersByBranch);

// For put Zone into multiple Branches 
router.post('/BranchesUtility', isAuth, BranchController.branchUtility);

// For transfer creditOfficer in Branch  
router.post('/transferCreditOfficer', isAuth, CreditOfficerController.transferCreditOfficer);

// router.delete('/Branches/:id', BranchController.deleteBranch);

module.exports = router;

