const express = require('express');
const router = express.Router();
const HRController = require('../Controllers/HR.controller');
const isAuth = require('../Middleware/isAuth');
const isPermission = require('../Middleware/isPermission');
const uploadFile = require('../Middleware/uploadFile');
const pagination = require('../Middleware/pagination');
const validation = require('../Middleware/validation');

// ======================== Asset Routes ============================

// For get list of all asset 
router.get('/assets/', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "asset", value: "list" }) }, pagination, HRController.getAllAsset);

// For get one asset
router.get('/asset/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "asset", value: "view" }) }, HRController.getAsset);

// For create asset
router.post('/asset', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "asset", value: "create" }) }, uploadFile.uploadAsset, validation.assetValidation, HRController.createAsset);

// For update asset
router.put('/asset/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "asset", value: "edit" }) }, uploadFile.uploadAsset, validation.assetValidation, HRController.updateAsset);

// For assign or return Asset
router.put('/assetManagement/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "asset", value: "edit" }) }, HRController.assetManagement);

// ===================================  get Employee Wise Case ===================================

router.get('/employeewiseCases/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "list" }) }, pagination, HRController.employeewiseCases)

// ===================================  Reminder Routes ===================================

// For get list of all reminder 
router.get('/reminders/', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "reminder", value: "list" }) }, pagination, HRController.getAllReminder);

// For get one reminder
router.get('/reminder/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "reminder", value: "view" }) }, HRController.getReminder);

// For create reminder
router.post('/reminder', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "reminder", value: "create" }) }, validation.reminderValidation, HRController.createReminder);

// For update reminder
router.put('/reminder/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "reminder", value: "edit" }) }, validation.reminderValidation, HRController.updateReminder);

// ===================================  Holiday Routes ===================================

// For get list of all holiday 
router.get('/holidays/', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "holiday", value: "list" }) }, pagination, HRController.getAllHoliday);

// For get one holiday
router.get('/holiday/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "holiday", value: "view" }) }, HRController.getHoliday);

// For create holiday
router.post('/holiday', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "holiday", value: "create" }) }, validation.holidayValidation, HRController.createHoliday);

// For update holiday
router.put('/holiday/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "holiday", value: "edit" }) }, validation.holidayValidation, HRController.updateHoliday);

// ===================================  Leave Routes ===================================

// For create leave
router.post('/leave', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "leave", value: "create" }) }, HRController.createLeave);

// For update leave
router.put('/leave/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "leave", value: "edit" }) }, HRController.updateLeave);


// ===================================  Attendance Routes ===================================

// For get list of all Attendances User Wise 
router.get('/attendances/:userId', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "holiday", value: "list" }) }, pagination, HRController.getAllAttendance);

module.exports = router;

