const express = require('express');
const router = express.Router();
const LeadController = require('../Controllers/Lead.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const validation = require('../Middleware/validation');
const pagination = require('../Middleware/pagination');
const isPermission = require('../Middleware/isPermission');

// For fetch leads 
router.get('/leads', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "list" }) }, pagination, LeadController.getAllLeads);
router.get('/leads/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "view" }) }, LeadController.getLead);

// For create new Lead
router.post('/leads', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "create" }) }, validation.leadValidation, LeadController.createLead);
router.post('/inquiry-leads', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "create" }) }, validation.inquiryLeadValidation, LeadController.createInquiryLead);

// For update  Lead
router.put('/leads/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "edit" }) }, validation.leadValidation, LeadController.updateLead);
router.put('/inquiry-leads/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "create" }) }, validation.inquiryLeadValidation, LeadController.updateInquiryLead);
router.put('/inquiry-leads/:id/addRemark', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "create" }) }, validation.addRemarkInquiryLead, LeadController.addRemarkInquiryLead);

// For convert lead into case 
router.put('/leads/:id/convertCase', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "lead", value: "edit" }) }, LeadController.convertCase);

module.exports = router;


