const express = require('express');
const router = express.Router();
const UtilitiesController = require('../Controllers/Utilities.controller');
const isAuth = require('../Middleware/isAuth');
const uploadFile = require('../Middleware/uploadFile');
const base64ToImage = require('../Middleware/base64toImage');
const MailController = require('../Controllers/Mail.controller');
const validation = require('../Middleware/validation');

// For generate PDF from uploaded Image
router.post('/generatePDF', isAuth, uploadFile.docs, UtilitiesController.imagestoPdf);

// For upload One Image
router.post('/uploadImage', isAuth, uploadFile.uploadImage, UtilitiesController.uploadImage);

// Get base64 String and upload it into Image
router.post('/base64Image', isAuth, base64ToImage.base64ToImage);

// For upload multiple Images i.e. used for upload composition Image and Employee Docs
router.post('/uploadImages', isAuth, uploadFile.uploadImages, UtilitiesController.uploadImages);

// For upload PDF 
router.post('/uploadDocuments', isAuth, uploadFile.uploadDocuments, UtilitiesController.uploadPDF);
router.post('/uploadBase64', isAuth, uploadFile.uploadDocuments, UtilitiesController.uploadBase64);


// For upload Empanelment Letters
// router.post('/uploadEmpanelment', isAuth, uploadFile.uploadEmpanelment, UtilitiesController.uploadPDF);

// For read Permissions JSON file
router.get('/readPermissionsFile', isAuth, UtilitiesController.readPermissionsFile);

router.get('/readDocumentsFile', isAuth, UtilitiesController.readDocumentsFile);

router.get('/updateEmpExpiryStatus', UtilitiesController.updateExpiryStatus);

router.post('/send-mail', isAuth, validation.mailSend, MailController.SendCustomMail)

router.get('/resize-cdn', UtilitiesController.resizeImage);

module.exports = router;


