const express = require('express');
const router = express.Router();

const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const uploadFile = require('../Middleware/uploadFile');
const pagination = require('../Middleware/pagination');
const isPermission = require('../Middleware/isPermission');
const validation = require('../Middleware/validation');

const nInvoiceController = require('../Controllers/nInvoice.controller');

router.get('/invoices/dashboard-count', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "Bill", value: "list" }) }, pagination, nInvoiceController.getDashboardInvoiceCount);

// // For get list of all Invoices
router.get('/invoices', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "Bill", value: "list" }) }, pagination, nInvoiceController.getAllInvoices);

// // For get only One Invoice
router.get('/invoices/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "Bill", value: "view" }) }, nInvoiceController.getInvoice);

// // For update Invoice
router.put('/invoices/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "Bill", value: "edit" }) }, uploadFile.uploadInvoice, validation.updateInvoiceValidation, nInvoiceController.updateInvoice);

// //For create new Invoice
router.post('/requestInvoices', isAuth, uploadFile.uploadInvoice, validation.invoiceCreateValidation, nInvoiceController.createInvoiceRequest);

// cancel old request and new request will be added
router.put('/cancel-invoice/:id', isAuth, nInvoiceController.cancelInvoice);
router.delete('/delete-invoice/:id', isAuth, nInvoiceController.deleteInvoice);

router.get('/merge-invoice', nInvoiceController.mergeInvoice);

router.get('/cases/:caseId/invoices', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "Bill", value: "list" }) }, nInvoiceController.invoicesByCase);


// // upload requested invoice
// router.put('/uploadInvoice/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "Bill", value: "edit" }) }, uploadFile.uploadInvoice, validation.uploadInvoiceValidation, InvoiceController.invoiceUpload);

module.exports = router;